@@ -210,6 +210,14 @@ def test_returning_kw_ends_where_clause():
     assert p.tokens[7].value == 'returning'
 
 
+def test_into_kw_ends_where_clause():  # issue324
+    s = 'select * from foo where a = 1 into baz'
+    p = sqlparse.parse(s)[0]
+    assert isinstance(p.tokens[8], sql.Where)
+    assert p.tokens[9].ttype == T.Keyword
+    assert p.tokens[9].value == 'into'
+
+
 @pytest.mark.parametrize('sql, expected', [
     # note: typecast needs to be 2nd token for this test
     ('select foo::integer from bar', 'integer'),
@@ -250,6 +258,16 @@ def test_grouping_alias_case():
     assert p.tokens[0].get_alias() == 'foo'
 
 
+def test_grouping_subquery_no_parens():
+    # Not totally sure if this is the right approach...
+    # When a THEN clause contains a subquery w/o parens around it *and*
+    # a WHERE condition, the WHERE grouper consumes END too.
+    # This takes makes sure that it doesn't fail.
+    p = sqlparse.parse('CASE WHEN 1 THEN select 2 where foo = 1 end')[0]
+    assert len(p.tokens) == 1
+    assert isinstance(p.tokens[0], sql.Case)
+
+
 def test_grouping_alias_returns_none():
     # see issue185
     p = sqlparse.parse('foo.bar')[0]
